"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createNodeGroup;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _kapellmeister = require("kapellmeister");

var _mergeKeys = _interopRequireDefault(require("../core/mergeKeys"));

var _types = require("../core/types");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var REGEX = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;

function kebabCase(str) {
  return str.replace(REGEX, function (match) {
    return '-' + match.toLowerCase();
  });
}

function createNodeGroup(getInterpolater) {
  var _class, _temp;

  var displayName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'NodeGroup';

  if (!getInterpolater || typeof getInterpolater !== 'function') {
    throw new Error('[resonance] getInterpolator function is required.');
  }

  var Node =
  /*#__PURE__*/
  function (_BaseNode) {
    _inherits(Node, _BaseNode);

    function Node(key, data) {
      var _this;

      _classCallCheck(this, Node);

      _this = _possibleConstructorReturn(this, _getPrototypeOf(Node).call(this));

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this)), "getInterpolator", getInterpolater);

      _this.key = key;
      _this.data = data;
      _this.type = _types.ENTER;
      return _this;
    }

    return Node;
  }(_kapellmeister.BaseNode);

  return _temp = _class =
  /*#__PURE__*/
  function (_Component) {
    _inherits(NodeGroup, _Component);

    function NodeGroup() {
      var _getPrototypeOf2;

      var _this2;

      _classCallCheck(this, NodeGroup);

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      _this2 = _possibleConstructorReturn(this, (_getPrototypeOf2 = _getPrototypeOf(NodeGroup)).call.apply(_getPrototypeOf2, [this].concat(args)));

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this2)), "ref", _react.default.createRef());

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this2)), "state", {
        nodeKeys: [],
        nodeHash: {},
        nodes: [],
        data: null
      });

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this2)), "animate", function () {
        var _this2$state = _this2.state,
            nodeKeys = _this2$state.nodeKeys,
            nodeHash = _this2$state.nodeHash;

        if (_this2.unmounting) {
          return;
        }

        var pending = false;

        for (var i = 0; i < nodeKeys.length; i++) {
          var k = nodeKeys[i];
          var n = nodeHash[k];
          var isTransitioning = n.isTransitioning();

          if (isTransitioning) {
            pending = true;
          }

          if (n.type === _types.LEAVE && !isTransitioning) {
            if (n.mounted === true) {
              n.unmount();
            }

            delete nodeHash[k];
            nodeKeys.splice(i, 1);
            i--;
          }
        }

        if (!pending) {
          _this2.interval.stop();
        }

        var _loop = function _loop(_i) {
          var k = nodeKeys[_i];
          var n = nodeHash[k];

          if (n.type === _types.ENTER && !n.mounted) {
            var parent = _this2.ref.current;

            var child = _this2.createChild(_this2.props.children, n, parent, k, _i);

            n.mounted = true;

            n.unmount = function () {
              parent.removeChild(child);
            };
          } else {
            n.updaters.forEach(function (updater) {
              updater.call(n, k, _i);
            });
          }
        };

        for (var _i = 0; _i < nodeKeys.length; _i++) {
          _loop(_i);
        }
      });

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this2)), "interval", null);

      _defineProperty(_assertThisInitialized(_assertThisInitialized(_this2)), "unmounting", false);

      return _this2;
    }

    _createClass(NodeGroup, [{
      key: "componentDidMount",
      value: function componentDidMount() {
        this.startInterval();
      }
    }, {
      key: "createChild",
      value: function createChild(template, node, parent, key, index) {
        var _this3 = this;

        var state = node.state,
            data = node.data;
        var nameSpace = this.ref.current.namespaceURI;
        var child = document.createElementNS(nameSpace, template.type);
        parent.appendChild(child);

        var _loop2 = function _loop2(prop) {
          var attr = kebabCase(prop);

          if (prop === 'children') {
            if (typeof template.props.children === 'function') {
              var value = template.props.children;
              var textNode = document.createTextNode(value(state, data, key, index));
              child.appendChild(textNode);
              node.updaters.push(function (k, i) {
                textNode.nodeValue = value(this.state, this.data, k, i);
              });
            } else {
              _react.default.Children.forEach(template.props.children, function (c) {
                _this3.createChild(c, node, child, key, index);
              });
            }
          } else if (prop === 'className') {
            child.setAttribute('class', template.props[prop]);
          } else if (typeof template.props[prop] === 'string' || typeof template.props[prop] === 'number') {
            child.setAttribute(attr, template.props[prop]);
          } else if (typeof template.props[prop] === 'function') {
            var _value = template.props[prop];

            if (prop.startsWith('on')) {
              child.addEventListener(prop.slice(2).toLowerCase(), _value.call(child, state, data, key, index));
            } else {
              child.setAttribute(attr, _value(state, data, key, index));
              node.updaters.push(function (k, i) {
                child.setAttribute(attr, _value(this.state, this.data, k, i));
              });
            }
          }
        };

        for (var prop in template.props) {
          _loop2(prop);
        }

        return child;
      }
    }, {
      key: "componentDidUpdate",
      value: function componentDidUpdate(prevProps) {
        if (prevProps.data !== this.props.data && !this.unmounting) {
          this.startInterval();
        }
      }
    }, {
      key: "startInterval",
      value: function startInterval() {
        if (!this.interval) {
          this.interval = (0, _kapellmeister.interval)(this.animate);
        } else {
          this.interval.restart(this.animate);
        }
      }
    }, {
      key: "componentWillUnmount",
      value: function componentWillUnmount() {
        var _this$state = this.state,
            nodeKeys = _this$state.nodeKeys,
            nodeHash = _this$state.nodeHash;
        this.unmounting = true;

        if (this.interval) {
          this.interval.stop();
        }

        nodeKeys.forEach(function (key) {
          nodeHash[key].stopTransitions();
        });
      }
    }, {
      key: "render",
      value: function render() {
        var _this$props = this.props,
            _this$props$wrapper = _this$props.wrapper,
            wrapper = _this$props$wrapper === void 0 ? 'div' : _this$props$wrapper,
            _this$props$wrapperCl = _this$props.wrapperClass,
            wrapperClass = _this$props$wrapperCl === void 0 ? '' : _this$props$wrapperCl,
            _this$props$wrapperSt = _this$props.wrapperStyle,
            wrapperStyle = _this$props$wrapperSt === void 0 ? {} : _this$props$wrapperSt;
        return _react.default.createElement(wrapper, {
          ref: this.ref,
          className: wrapperClass,
          style: wrapperStyle
        });
      }
    }], [{
      key: "getDerivedStateFromProps",
      value: function getDerivedStateFromProps(nextProps, prevState) {
        if (nextProps.data !== prevState.data) {
          var data = nextProps.data,
              keyAccessor = nextProps.keyAccessor,
              start = nextProps.start,
              enter = nextProps.enter,
              update = nextProps.update,
              leave = nextProps.leave;
          var nodeKeys = prevState.nodeKeys,
              nodeHash = prevState.nodeHash;
          var keyIndex = {};

          for (var i = 0; i < nodeKeys.length; i++) {
            keyIndex[nodeKeys[i]] = i;
          }

          var nextKeyIndex = {};
          var nextNodeKeys = [];

          for (var _i2 = 0; _i2 < data.length; _i2++) {
            var d = data[_i2];
            var k = keyAccessor(d, _i2);
            nextKeyIndex[k] = _i2;
            nextNodeKeys.push(k);

            if (keyIndex[k] === undefined) {
              var node = new Node(k, d);
              node.updaters = [];
              nodeHash[k] = node;
            }
          }

          for (var _i3 = 0; _i3 < nodeKeys.length; _i3++) {
            var _k = nodeKeys[_i3];
            var n = nodeHash[_k];

            if (nextKeyIndex[_k] !== undefined) {
              n.data = data[nextKeyIndex[_k]];
              n.type = _types.UPDATE;
            } else {
              n.type = _types.LEAVE;
            }
          }

          var mergedNodeKeys = (0, _mergeKeys.default)(nodeKeys, keyIndex, nextNodeKeys, nextKeyIndex);

          for (var _i4 = 0; _i4 < mergedNodeKeys.length; _i4++) {
            var _k2 = mergedNodeKeys[_i4];
            var _n = nodeHash[_k2];
            var _d = _n.data;

            if (_n.type === _types.ENTER) {
              _n.setState(start(_d, nextKeyIndex[_k2]));

              _n.transition(enter(_d, nextKeyIndex[_k2]));
            } else if (_n.type === _types.LEAVE) {
              _n.transition(leave(_d, keyIndex[_k2]));
            } else {
              _n.transition(update(_d, nextKeyIndex[_k2]));
            }
          }

          return {
            data: data,
            nodeHash: nodeHash,
            nodeKeys: mergedNodeKeys
          };
        }

        return null;
      }
    }]);

    return NodeGroup;
  }(_react.Component), _defineProperty(_class, "displayName", displayName), _defineProperty(_class, "propTypes", {
    wrapper: _propTypes.default.string,
    wrapperClass: _propTypes.default.string,
    wrapperStyle: _propTypes.default.object,
    data: _propTypes.default.array.isRequired,
    keyAccessor: _propTypes.default.func.isRequired,
    start: _propTypes.default.func.isRequired,
    enter: _propTypes.default.func,
    update: _propTypes.default.func,
    leave: _propTypes.default.func,
    children: _propTypes.default.oneOfType([_propTypes.default.object, _propTypes.default.func]).isRequired
  }), _defineProperty(_class, "defaultProps", {
    enter: function enter() {},
    update: function update() {},
    leave: function leave() {}
  }), _temp;
}